//////////////////////////////////////////////////////////////
//
//	Copyright Artistic Licence (UK) Ltd 1999-2000
//	Author:	WDH
//	Email:	Support@ArtisticLicence.com
//
//	This file contains all key defines and structures
//	 for Art-Net
//
//
//////////////////////////////////////////////////////////////

#ifndef __ARTLIC_ART_NET_H__
#define __ARTLIC_ART_NET_H__


#define uchar unsigned char
#define ushort unsigned short int
#define ulong unsigned int
// 8, 16, 32 bit fields


#define NETID0 2
	// NetID when DIP Sw #4 =OFF
#define NETID1 126
	// NetID when DIP Sw #4 =ON

#define DefaultPort 0x1936

#define ProtocolVersion 14
	// DMX-Hub protocol version.


// OEM Codes (Byte swapped for net order)

#define OemDMXHub      	0x0000
#define OemNetgate	0x0001
#define OemMAHub       	0x0002
#define OemStageHub	0x0003
#define OemLewLightCv2	0x0004

#define OemNetgateXT	0x8000
#define OemNetPatch	0x8001
#define OemDMXHubXT	0x8002

// OpCodes (byte-swapped)

#define OpPoll   	0x2000 /* Poll */
#define OpPollReply   	0x2100 /* ArtPollReply */
#define OpOutput 	0x5000 /* Output */
#define OpAddress 	0x6000 /* Program Node Settings */
#define OpInput 	0x7000 /* Setup DMX input enables */
#define OpVideoSetup 	0xa010 /* Setup video scan and font */
#define OpVideoPalette 	0xa020 /* Setup colour palette */
#define OpVideoData 	0xa040 /* Video Data */


// Video Specific Defines

#define MaxWinFontChar  64      // Max char in windows font name
#define MaxFontChar     256     // Max char in a soft font
#define MaxFontHeight   16      // Max scan lines per char
#define MaxPalette      17      // Max colour entry in palette
#define MaxVideoX       80      // Max Char across screen
#define MaxVideoY       50      // Max Lines down screen




// Report Codes (Status reports)

#define RcDebug		0x0000	// Booted in debug mode (Only used in development 
#define RcPowerOk	0x0001	// Power On Tests successful
#define RcPowerFail	0x0002	// Hardware tests failed at Power On
#define RcSocketWr1	0x0003	// Last UDP from Node failed due to truncated length,
				//  Most likely caused by a collision.
#define RcParseFail	0x0004	// Unable to identify last UDP transmission.
				// Check OpCode and packet length
#define RcUdpFail	0x0005	// Unable to open Udp Socket in last transmission attempt
#define RcShNameOk	0x0006	// Confirms that Short Name programming via ArtAddress accepted
#define RcLoNameOk	0x0007	// Confirms that Long Name programming via ArtAddress accepted
#define RcDmxError	0x0008	// DMX512 receive errors detected
#define RcDmxUdpFull	0x0009	// Ran out of internal DMX transmit buffers
				//  Most likely caused by receiving ArtDmx
				//  packets at a rate faster than DMX512 transmit
				//  speed
#define RcDmxRxFull	0x000a	// Ran out of internal DMX rx buffers
#define RcSwitchErr	0x000b	// Rx Universe switches conflict




typedef struct {
	uchar IP[4];	// IP address
	ushort Port;	// UDP port	BYTE-SWAPPED MANUALLY
} T_Addr;

// Poll request from Server to Node.

typedef struct S_ArtPoll {
	uchar ID[8];                    // protocol ID = "Art-Net"
	ushort OpCode;                  // == OpPoll
	uchar VersionH;                 // 0
	uchar Version;                  // protocol version, currently 14
	uchar TalkToMe;                 // bit 0 = 0 if reply is broadcast
                                        // bit 0 = 1 if reply is to server IP
                                        // bit 1 = 0 then Node only replies when polled
                                        // bit 1 = 1 then Node sends reply when it needs to
	uchar Pad;
} T_ArtPoll;

//The DMX-Hub records the Server's IP:Port for reply.
//The DMX-Hub now returns a packet to IP:Port (on the Server's subnet)
//as follows:

#define MaxNumPorts 4
#define ShortNameLength 18
#define LongNameLength 64

typedef struct S_ArtPollReply {
	uchar ID[8];                    // protocol ID = "Art-Net"
	ushort OpCode;                  // == OpPollReply
	T_Addr BoxAddr;                 // 0 if not yet configured

	uchar VersionInfoH;             // The node's current FIRMWARE VERS hi
	uchar VersionInfo;              // The node's current FIRMWARE VERS lo

	uchar SubSwitchH;               // 0 - not used yet
	uchar SubSwitch;                // from switch on front panel (0-15)
					//

	uchar OemH;
        uchar Oem;			// Manufacturer code, bit 15 set if
					// extended features avail

        ushort EstaManH;		// Reserved for ESTA manufacturer id hi, zero for now
        ushort EstaMan;			// Reserved for ESTA manufacturer id lo, zero for now

	uchar ShortName[ShortNameLength]; // short name defaults to IP
	uchar LongName[LongNameLength];   // long name
	uchar NodeReport[LongNameLength]; // Text feedback of Node status or errors
					  //  also used for debug info

	uchar NumPortsH;                // 0
	uchar NumPorts;                 // 4 If num i/p ports is dif to output ports, return biggest

	uchar PortTypes[MaxNumPorts];   // bit 7 is output
                                        // bit 6 is input
					// bits 0-5 are protocol number (0= DMX, 1=MIDI)
                                        // for DMX-Hub ={0xc0,0xc0,0xc0,0xc0};


        uchar GoodInput[MaxNumPorts];  	// bit 7 is good data received
                                        // bit 6 is data includes test packets
                                        // bit 5 is data includes SIP's
					// bit 4 is data includes text
					// bit 3 set is input is disabled
					// bit 2-0 not used, transmitted as zero.
					// Don't test for zero!

	uchar GoodOutput[MaxNumPorts]; 	// bit 7 is data is transmitting
					// bit 6 is data includes test packets
					// bit 5 is data includes SIP's
					// bit 4 is data includes text
					// bit 3-0 not used, transmitted as zero.
					// Don't test for zero!

        uchar Swin[MaxNumPorts];   	// Low nibble is the port input
					// address wheel for this port. On
					// DMX-Hub, this is the front panel
					//  rotary switch.
					// Hi Nibble is the Sub-net
					// This byte shows to which of 256 universes, the
					// data belongs


	uchar Swout[MaxNumPorts];   	// Low nibble is the port output
					// address wheel for this port. On
					// DMX-Hub, this is the front panel
					//  rotary switch.
					// Hi Nibble is the Sub-net
					// This byte shows to which of 256 universes, the
					// data belongs

	uchar SwVideo;		   	// Low nibble is the value of the video
					// output channel

	uchar SwMacro;		   	// Bit 0 is Macro input 1
					// Bit 7 is Macro input 8
					// Only implemented if OEM bit 15 set.
					// Netgate XT contains this implementation
					// Any change in state of a bit will
					//  cause a transmission.
					// Server should not assume that only
					//  one bit has changed.
					// The Node is responsible for debounce

        uchar SwRemote;		   	// Bit 0 is Remote input 1
					// Bit 7 is Remote input 8
					// Only implemented if OEM bit 15 set.
					// Netgate XT contains this implementation
					// Any change in state of a bit will
					//  cause a transmission.
					// Server should not assume that only
					//  one bit has changed.
					// The Node is responsible for debounce



} __attribute__((packed)) T_ArtPollReply;


// The following packet is used to transfer DMX to and from the Node

#define MaxDataLength 512

typedef struct S_ArtDmx {
	uchar ID[8];                    // protocol ID = "Art-Net"
	ushort OpCode;                  // == OpOutput
	uchar VersionH;                 // 0
	uchar Version;                  // protocol version, starts at 0
	uchar Filler1;			// TalkToMe position in ArtPoll
	uchar Physical;                 // The physical i/p 0-3. For Debug only
	ushort Universe;                // hi nib = subnet, lo nib = wheel position
	ushort Length;			// BYTE-SWAPPED MANUALLY
	uchar Data[MaxDataLength];
} __attribute__((packed)) T_ArtDmx;

// The following packet is used to transfer settings from server to node


typedef struct S_ArtAddress {
	uchar ID[8];                    // protocol ID = "Art-Net"
	ushort OpCode;                  // == OpAddress
	uchar VersionH;                 // 0
	uchar Version;                  // protocol version, starts at 0
	uchar Filler1;			// TalkToMe position in ArtPoll
	uchar Filler2;                  // The physical i/p 0-3. For Debug only


	uchar ShortName[ShortNameLength]; // null ignored
	uchar LongName[LongNameLength];   // null ignored
	uchar Swin[MaxNumPorts];   	// Low nibble is the port input
					// address wheel for this port. On
					// DMX-Hub, this is the front panel
					//  rotary switch.
					// Bit 7 hi = use data
					// Server gets array size from ArtPoll
					//  reply

	uchar Swout[MaxNumPorts];   	// Low nibble is the port output
					// address wheel for this port. On
					// DMX-Hub, this is the front panel
					//  rotary switch.
					// Bit 7 hi = use data
					// Server gets array size from ArtPoll
					//  reply

	uchar SwSub;		   	// Low nibble is the value of the sub-net
					// switch
					// Bit 7 hi = use data

	uchar SwVideo;		   	// Low nibble is the value of the video
					// output channel
					// Bit 7 hi = use data



} __attribute__((packed)) T_ArtAddress;


typedef struct S_ArtInput {
	uchar ID[8];                    // protocol ID = "Art-Net"
	ushort OpCode;                  // == OpInput
	uchar VersionH;                 // 0
	uchar Version;                  // protocol version, starts at 0
	uchar Filler1;			// 0
	uchar Filler2;                  // 0
	uchar NumPortsH;                // 0
	uchar NumPorts;                 // 4 If num i/p ports is dif to output ports, return biggest


	uchar Input[MaxNumPorts];   	// Bit 0 = 0 to enable dmx input.
                                        // Bit 0 = 1 to disable input
                                        //  Power on default is enabled
                                        // Use NumPorts in ArtPollReply for array size.


} __attribute__((packed)) T_ArtInput;




typedef struct S_ArtVideoSetup {
	uchar ID[8];                    // protocol ID = "Art-Net"
	ushort OpCode;                  // == OpVideoSetup
	uchar VersionH;                 // 0
	uchar Version;                  // protocol version, starts at 0
	uchar Filler1;			// 0
	uchar Filler2;                  // 0

	uchar Filler3;                  // 0
        uchar Filler4;                  // 0
	uchar Control;                  // 0 = local video, 1 = use ArtVideoData

        uchar FontHeight;               // Scan lines per char. 8-16 valid

        uchar FirstFont;                // First font to program  0-255

        uchar FontRange;                // Number fonts to program  1-(255-FirstFont)

	char WinFontName[MaxWinFontChar];
                                        // nearest windows font, null if not used
	uchar FontData[MaxFontChar*MaxFontHeight];
                                   	// font data, array defined to max size


} __attribute__((packed)) T_ArtVideoSetup;


typedef struct S_ArtVideoPalette {
	uchar ID[8];                    // protocol ID = "Art-Net"
	ushort OpCode;                  // == OpVideoPalette
	uchar VersionH;                 // 0
	uchar Version;                  // protocol version, starts at 0
	uchar Filler1;			// 0
	uchar Filler2;                  // 0
	uchar ColourRed[MaxPalette];    // New red values, 0x00 - 0x3f
	uchar ColourGreen[MaxPalette];  // New green values, 0x00 - 0x3f
	uchar ColourBlue[MaxPalette];   // New blue values, 0x00 - 0x3f


} __attribute__((packed)) T_ArtVideoPalette;


typedef struct S_ArtVideoData {
	uchar ID[8];                    // protocol ID = "Art-Net"
	ushort OpCode;                  // == OpVideoData
	uchar VersionH;                 // 0
	uchar Version;                  // protocol version, starts at 0
	uchar Filler1;			// 0
	uchar Filler2;                  // 0
	uchar PosX;                     // Start X Coord, 0 - VideoColumns-1
        uchar PosY;                     // Start Y Coord, 0 - VideoLines-1
        uchar LenX;                     // Number of char in each line, 0 - VideoColums-PosX
        uchar LenY;                     // Number of lines, 0 - VideoLines-PosY


        ushort Data[MaxVideoX*MaxVideoY];
                                        // New character data

} __attribute__((packed)) T_ArtVideoData;


#endif
