/*
 * dmxoe_codec.c
 */

#include "dmxoe_codec.h"


#undef DEBUG

#ifdef DEBUG
# define DEBUG_FUNC(x) x
#else
# define DEBUG_FUNC(x)
#endif


class ShownetCodec
{
private:
  U16 ThisVersion = 1;

  U16 packetSequence = 1;

  U32 my_ip = (192<<24) | (168<<16) | (0<<8) | 49;


public:
  ShownetCodec ()
  {
  }

  ~ShownetCodec ()
  {
  }

  U16 getPacketSequence()
  {
    return packetSequence++;
  }


/*
 *   1-100 slots     one block contains all the slots
 * 101-200 slots     two blocks: the slots are split evenly between
 *                   them with the first block getting the extra.
 *                   eg. for 121 slots the first block would contain
 *                   the first 61 slots, and the second block would
 *                   contain the last 60 slots.
 *     201 slots     first block has 51 slots, second block has 50 slots,
 *                   third block has 100 slots
 *     202 slots     first block has 51 slots, second bock has 50 slots,
 *                   third block has 50 slots, fourh block has 51 slots.
 *     203 slots     51, 50, 51, 51
 *     204 slots     51, 51, 51, 51
 *     205 slots     52, 51, 51, 51
 *     206 slots     52, 51, 51, 52
 *     207 slots     52, 51, 52, 52
 *     208 slots     52, 52, 52, 52
 *     209 slots     53, 52, 52, 52
 *     210 slots     53, 52, 52, 53
 *     and so on for the rest.
 */
  long getSlotLayout (int count, U16 *lengths)
  {
    U16 c[4];

    memset (c, 0, sizeof(c));

    if (count>=1 && count<=100)
      {
        DEBUG_FUNC(printf ("case1\n"));
        c[0] = count;
      }
    else if (count>=101 && count<=200)
      {
        DEBUG_FUNC(printf ("case2\n"));
        c[1] = count/2;
        c[0] = count - c[1];
      }
    else if (count==201)
      {
        DEBUG_FUNC(printf ("case3\n"));
        c[0] = 51;
        c[1] = 50;
        c[2] = 100;
      }
    else if (count >= 202)
      {
        register int t1 = count/4;
        register int t2 = count%4;
        c[0] = t1 + ((t2>0)?1:0);
        c[3] = t1 + ((t2>1)?1:0);
        c[2] = t1 + ((t2>2)?1:0);
        c[1] = t1;
        DEBUG_FUNC(printf ("case4\n"));
      }

    if (lengths)
      memcpy (lengths, c, sizeof(U16)*4);

    return (c[3]<<24) | (c[2]<<16) | (c[1]<<8) | c[0];
  }


/*
 *
 *
 */
  size_t rle_encode (U8 *dstbuf, size_t dstlen, U8 *srcbuf, size_t srclen)
  {
    size_t cnt = 0;
    size_t len = srclen;

    /* dstlen = ((srclen / 0x7f) * (0x7f+1)) */
    if (srclen + (srclen/0x7f) > dstlen)
      {
        DEBUG_FUNC(printf ("rle_encode: error, target to small src=%i dst=%i\n", srclen, dstlen));
        return -1;
      }

    while (len>0)
      {
        *(dstbuf++) = (len&0x7f);
        memcpy (dstbuf, srcbuf, len&0x7f);
        dstbuf += len&0x7f;
        srcbuf += len&0x7f;
        cnt += len&0x7f;
        len -= len&0x7f;
      }
    return cnt;
  }




  size_t rle_decode (U8 srcbuf[], size_t srclen, U8 dstbuf[], size_t dstlen)
  {
    U8  count = 0;
    int src   = 0;
    int dst   = 0;

    if(srclen <= 2 && (srcbuf[0] & 0x80) == 0)
      while(src<srclen && dst<dstlen)
        dstbuf[dst++] = srcbuf[src++];
    else
      while(src<srclen && dst<dstlen)
        {
          count = srcbuf[src++];
          if(count & 0x80)
            {
              count &= ~0x80;
              memset(&dstbuf[dst], srcbuf[src], count);
              dst += count;
              src++;
            }
          else
            {
              memcpy(&dstbuf[dst], &srcbuf[src], count);
              dst += count;
              src += count;
            }
        }
    return dst;
  }



  size_t shownet_encode (U8 dmxbuf[], size_t dmxlen, U8 netbuf[], size_t netlen)
  {
    header *phead = (header *)netbuf;
    int     rle_offset=0;
    int     i;

    if (sizeof(header) != HEADER_SIZE)
      {
        DEBUG_FUNC(printf("FATAL: illegal size of paket-structure\n"));
        return -1;
      }

    if (dmxlen <= 0)
      return 0;

    phead->version  = ThisVersion;
    phead->srcip    = htonl(my_ip);
    phead->sequence = htons(getPacketSequence());
    phead->long_sig0[0] = 0;
    phead->long_sig0[1] = 0;
    phead->long_sig0[2] = 0x58; /* 0x258 = 600 (zufall?) */
    phead->long_sig0[3] = 2;
    memcpy(phead->devicename, "console1", 8);
    phead->long_sig1[0] = 0;
    //  rle encoded dimmer data

    GetSlotLayout (dmxlen, phead->length);

    for (i = 0; i<4; i++)
      {
  /*      phead->length[i] = rle_length; */

        phead->slot[i] = 1 + rle_offset;
        phead->rle_offset[i] = RLE_OFFSET_BIAS + rle_offset;
        phead->rle_offset[i+1] = phead->rle_offset[i] + phead->length[i];

        rle_encode (phead->data + rle_offset,
                    (netlen-sizeof(header)) / 4,

                    dmxbuf + phead->slot[i] - 1,
                    phead->length[i]
                   );

        rle_offset += phead->length[i];
      }
    return HEADER_SIZE + rle_offset;
  }



  size_t shownet_decode (U8 pktbuf[], size_t pktlen, U8 netbuf[], size_t netlen)
  {
    header *phead = (header*) pktbuf;
    int     rle_offset,
            rle_length;
    int     i;
    size_t  len = 0;

    for (i = 0; i<4; i++)
      {
        size_t t=0;
        if(phead->slot[i] == 0) break;
        rle_offset = phead->rle_offset[i] - RLE_OFFSET_BIAS;
        rle_length = phead->rle_offset[i+1] - phead->rle_offset[i];
        t = rle_decode (phead->data + rle_offset,
                    rle_length,
                    netbuf + phead->slot[i] - 1,
                    -1
                   );
        if (t>0 &&  phead->slot[i]-1+t> len)
          len = phead->slot[i]-1+t;
      }
    return len;
  }

};



